pub struct EigDecomp<S, D0: Dim, D1: Dim> {
pub eigenvalues: Array<S, (D0,)>,
pub left_eigenvectors: Option<Array<S, (D0, D1)>>,
pub right_eigenvectors: Option<Array<S, (D0, D1)>>,
}Expand description
Holds the results of a general eigenvalue decomposition.
The scalar type S is the backend’s spectral scalar for the input matrix
scalar. For a real matrix this is typically a complex scalar, while for a
complex matrix it is usually the matrix scalar itself.
Fields
eigenvalues: Array<S, (D0,)>left_eigenvectors: Option<Array<S, (D0, D1)>>right_eigenvectors: Option<Array<S, (D0, D1)>>Auto Trait Implementations
impl<S, D0, D1> Freeze for EigDecomp<S, D0, D1>
impl<S, D0, D1> RefUnwindSafe for EigDecomp<S, D0, D1>where
<D0 as Dim>::Buffer<S, (), Global>: RefUnwindSafe,
<D0 as Dim>::Buffer<S, (D1,), Global>: RefUnwindSafe,
impl<S, D0, D1> Send for EigDecomp<S, D0, D1>
impl<S, D0, D1> Sync for EigDecomp<S, D0, D1>
impl<S, D0, D1> Unpin for EigDecomp<S, D0, D1>
impl<S, D0, D1> UnsafeUnpin for EigDecomp<S, D0, D1>where
<D0 as Dim>::Buffer<S, (), Global>: UnsafeUnpin,
<D0 as Dim>::Buffer<S, (D1,), Global>: UnsafeUnpin,
impl<S, D0, D1> UnwindSafe for EigDecomp<S, D0, D1>where
<D0 as Dim>::Buffer<S, (), Global>: UnwindSafe,
<D0 as Dim>::Buffer<S, (D1,), Global>: UnwindSafe,