pub struct SVDDecomp<T, S, D: Dim> {
pub s: Array<S, (D,)>,
pub u: Array<T, (D, D)>,
pub vt: Array<T, (D, D)>,
}Expand description
Holds the results of a singular value decomposition, including singular values and the left and right singular vectors.
T is the matrix scalar type, S is the singular-value scalar type,
and D is the matrix dimension type.
Fields
s: Array<S, (D,)>u: Array<T, (D, D)>vt: Array<T, (D, D)>Auto Trait Implementations
impl<T, S, D> Freeze for SVDDecomp<T, S, D>
impl<T, S, D> RefUnwindSafe for SVDDecomp<T, S, D>where
<D as Dim>::Buffer<S, (), Global>: RefUnwindSafe,
<D as Dim>::Buffer<T, (D,), Global>: RefUnwindSafe,
impl<T, S, D> Send for SVDDecomp<T, S, D>
impl<T, S, D> Sync for SVDDecomp<T, S, D>
impl<T, S, D> Unpin for SVDDecomp<T, S, D>
impl<T, S, D> UnsafeUnpin for SVDDecomp<T, S, D>where
<D as Dim>::Buffer<S, (), Global>: UnsafeUnpin,
<D as Dim>::Buffer<T, (D,), Global>: UnsafeUnpin,
impl<T, S, D> UnwindSafe for SVDDecomp<T, S, D>where
<D as Dim>::Buffer<S, (), Global>: UnwindSafe,
<D as Dim>::Buffer<T, (D,), Global>: UnwindSafe,